<?php declare(strict_types=1);

/*
 * Copyright (c) 2020  https://rewiv.com sikofitt@gmail.com
 *
 * This file is a part of Olive BBS
 *
 * This Source Code Form is subject to the
 * terms of the Mozilla Public License, v. 2.0.
 *
 * If a copy of the MPL was not distributed with this file,
 * You can obtain one at https://mozilla.org/MPL/2.0/.
 *
 *         ___                   ___    ___
 *        (   ).-.              (   )  (   )
 *    .--. | |( __)___  ___ .--. | |.-. | |.-.     .--.
 *   /    \| |(''"(   )(   /    \| /   \| /   \  /  _  \
 *  |  .-. | | | | | |  | |  .-. |  .-. |  .-. |. .' `. ;
 *  | |  | | | | | | |  | |  | | | |  | | |  | || '   | |
 *  | |  | | | | | | |  | |  |/  | |  | | |  | |_\_`.(___)
 *  | |  | | | | | | |  | |  ' _.| |  | | |  | (   ). '.
 *  | '  | | | | | ' '  ; |  .'.-| '  | | '  | || |  `\ |
 *  '  `-' | | | |  \ `' /'  `-' ' `-' ;' `-' ; ; '._,' '
 *   `.__.(___(___)  '_.'  `.__.' `.__.  `.__.   '.___.'
 *
 */

namespace Olivebbs\Map;

use Olivebbs\Map\Enum\KeyType;
use Olivebbs\Map\Enum\ValueType;
use Olivebbs\Map\Exception\ImmutableMapException;

class ImmutableMap extends GenericMap
{
    private function __construct(
        KeyType $keyType,
        object|string $valueType,
        array $values
    ) {
        parent::__construct(keyType: $keyType, valueType: $valueType);

        $this->assertInitialValues(initialValues: $values);
        $this->map->putAll(values: $values);
    }

    public function offsetSet(mixed $offset, mixed $value): void
    {
        throw new ImmutableMapException(message: sprintf('Cannot change values in %s', __CLASS__));
    }

    public function offsetUnset(mixed $offset): void
    {
        throw new ImmutableMapException(message: sprintf('Cannot unset values in %s', __CLASS__));
    }

    public static function create(KeyType $keyType, ValueType $valueType, array $values): static
    {
        return new static(keyType: $keyType, valueType: $valueType, values: $values);
    }
}
